package ACL2::Lisp::Program;
use base 'ACL2::Lisp::BaseNode';

=head1 NAME

ACL2::Lisp::Program - Global node for the whole Lisp program

=head1 FUNCTIONS

This module inherits from ACL2::Lisp::BaseNode, and defines the
following new methods:

=cut

#
#  Global node for the whole Lisp program
#  Copyright (C) 2009 Antonio García Domínguez
#  $Id$
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

sub new {
    my ($pkg, $rv_value) = @_;

    my (@elems) = map { $_->source } @$rv_value;
    my $source  = join '', @elems;

    return ACL2::Lisp::BaseNode::new(__PACKAGE__, $rv_value, $source);
}

sub clean_tree {
    my ($self)   = @_;
    my @elems    = map { $_->clean_tree } @{$self->value};
    return __PACKAGE__->new(\@elems);
}

=head2 comment_count($self)

Returns the number of top-level comments in this program (comments
that are direct descendants of this node).

=cut

sub comment_count {
    my ($self) = @_;
    my @exprs = grep { $_->type eq "comment" } @{$self->value};
    return scalar @exprs;
}

=head2 expression_count($self)

Returns the number of top-level expressions in this program (direct
descendants of this node, excluding whitespace and comment nodes).

=cut

sub expression_count {
    my ($self) = @_;

    my @exprs = grep {
        $_->type ne "whitespace" && $_->type ne "comment"
    } @{$self->value};

    return scalar @exprs;
}

1;
