package ACL2::Orden::DefinitionCategory;
use base ACL2::Orden;

=head1 NAME

  ACL2::Orden::DefinitionCategory - events with name, args and body

=head1 FUNCTIONS

=cut

#
#    DefinitionCategory: events with name, args and body
#    Copyright (C) 2006-2009 Antonio García Domínguez
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Localizacion;
use ACL2::Tags;
my $lh = ACL2::Localizacion->get_handle;

my %KNOWN_CMDS = (
    "defmacro" => ACL2::Tags::DEFMACRO_TAG,
    "defabbrev" => ACL2::Tags::DEFABBREV_TAG
);

sub protegido_inicializar_sexpr
{
    my ($this) = @_;

    # Element name: must be in KNOWN_CMDS
    my $cmd = lc $this->get_lisp->value->[0]->value;
    $cmd    =~ s/\W//g;
    die $lh->maketext("Categoría [_1] equivocada: no es uno de [_2]",
                      __PACKAGE__, join(",", keys %KNOWN_CMDS))
        unless exists $KNOWN_CMDS{$cmd};
    $this->{tag} = $KNOWN_CMDS{$cmd};

    # Especificamos los atributos existentes, con valores por defecto
    $this->{nombre}  = $this->get_lisp->value->[1]->value;
    $this->{nombre} =~ tr/a-z/A-Z/;
    $this->{args}    = $this->get_lisp->value->[2];
    $this->{formula} = $this->get_lisp_source;
}

sub protegido_inicializar_salida {}

sub protegido_cabecera
{
    my ($orden, $writer) = @_;
    $writer->startTag($orden->{tag},
		      "name" => $orden->{nombre},
		      "args" => $orden->{args}->source,
		      "formula" => $orden->{formula},
		      "package" => $orden->get_paquete);
}

sub protegido_cuerpo {}

1;
