package ACL2::Orden::Includebook;
use base ACL2::Orden;

#
#    Orden INCLUDE-BOOK
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Tags;
use ACL2::Localizacion;
use Carp;

my $lh = ACL2::Localizacion->get_handle;

=head2 get_nombre_libro($this)

Uso: ME<eacute>todo pE<uacute>blico.
Responsabilidades: Devuelve el nombre corto (ruta no completa) del
libro incluido.
Entrada: Ninguna.
Salida: Cadena con el nombre corto del libro incluido en la orden.

=cut

sub get_nombre_libro
{
    return $_[0]->{nombre_libro};
}

=head2 get_ruta_libro

Uso: ME<eacute>todo pE<uacute>blico.

Responsabilidades: Obtiene la ruta a travE<eacute>s de la cual se ha
de acceder al libro.

Excepciones: El libro no es accesible con la misma
capitalizaciE<oacute>n o en minE<uacute>sculas de la ruta relativa.

Devuelve: Ruta absoluta a travE<eacute>s de la cual se debe acceder al
libro.

=cut

sub get_ruta_libro
{
    my ($self, $volumen, $dirBase) = @_;

    # Al ser un include-book, accedemos a través del portcullis
    my $ruta_completa = File::Spec->catpath(
        $volumen, $dirBase, $self->get_nombre_libro . ".lisp");
    $ruta_completa = File::Spec->catpath(
        $volumen, $dirBase, lc $self->get_nombre_libro . ".lisp")
        if ! -f $ruta_completa;

    # No se encontró el libro
    croak $lh->maketext(
        "No se pudo localizar el fichero de certificación del libro [_1]",
        $self->get_nombre_libro)
        if ! -f $ruta_completa;

    return $ruta_completa;
}

sub protegido_inicializar_sexpr
{
    # Retiramos las comillas
    my $nombreLibro       = $_[0]->get_lisp->value->[1]->value;
    $nombreLibro          =~ s/\"//g;
    $_[0]->{nombre_libro} = $nombreLibro;
}

sub protegido_inicializar_salida
{
    my ($this) = @_;

    # Intentamos obtener la ruta al LISP
    #
    # Esto no siempre funciona: no se muestra dicha cadena cuando se
    # hace la inclusión desde un CERTIFY-BOOK. En dicho caso, nos
    # limitamos a añadir .lisp al nombre del libro.
    my ($ruta_objeto) = ${$this->get_salida} =~ /Loading (.*?)\.o/;

    my ($vol, $dir, $fich) = File::Spec->splitpath($ruta_objeto
                                                || $this->get_nombre_libro);

    # Probamos con minúsculas y mayúsculas
    my $ruta_libro  = File::Spec->catpath($vol, $dir, $fich . ".lisp");
    $ruta_libro  = File::Spec->catpath($vol, lc $dir, lc $fich . ".lisp")
        if ! -f $ruta_libro;

    # Buscamos el archivo de certificado .acl2 también
    my $ruta_cert = File::Spec->catpath($vol, $dir, $fich . ".acl2");
    $ruta_cert = File::Spec->catpath($vol, lc $dir, lc $fich . ".acl2")
        if ! -f $ruta_cert;
    $ruta_cert = File::Spec->catpath($vol, $dir, "cert.acl2")
        if ! -f $ruta_cert;

    # Comprobamos que ambas rutas existen
    croak $lh->maketext("No se pudo localizar el libro [_1]",
                        $ruta_libro)
        if ! -f $ruta_libro;
    croak $lh->maketext("No se pudo localizar el fichero de certificación del libro [_1]",
                        $ruta_cert)
        if ! -f $ruta_cert;

    $this->{ruta_libro} = $ruta_libro;
    $this->{ruta_cert}  = $ruta_cert;
}

sub protegido_cabecera
{
    $_[1]->startTag(ACL2::Tags::INCLUDEBOOK_TAG,
                    "name" => $_[0]->{nombre_libro},
                    "path_book" => File::Spec->abs2rel($_[0]->{ruta_libro}),
                    "path_cert" => File::Spec->abs2rel($_[0]->{ruta_cert}));
}

sub protegido_cuerpo
{
    my ($this, $writer) = @_;
    ACL2::Orden->salida_parrafos($writer, ${$this->get_salida});
}

1;
